import { Fragment as _Fragment, jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
/*
 * Copyright (c) 2021-2023 Datalayer, Inc.
 *
 * MIT License
 */
import { useState, useEffect } from 'react';
import { createPortal } from 'react-dom';
import { Box } from '@primer/react';
import { useJupyter } from './../../jupyter';
import { asObservable, Lumino } from '../lumino';
import { CellMetadataEditor } from './cell/metadata';
import { newUuid } from '../../utils';
import { useNotebookStore } from './NotebookState';
import { NotebookAdapter } from './NotebookAdapter';
import './Notebook.css';
/**
 * This component creates a Notebook as a collection of cells
 * with sidebars.
 *
 * @param props The notebook properties.
 * @returns A Notebook React.js component.
 */
export const Notebook = (props) => {
    const { serviceManager, defaultKernel, lite } = useJupyter({
        lite: props.lite,
        serverless: props.serverless,
        serviceManager: props.serviceManager,
        useRunningKernelId: props.useRunningKernelId,
        useRunningKernelIndex: props.useRunningKernelIndex,
    });
    const { Toolbar, extensions, height, maxHeight, nbformat, nbgrader, path, readonly, serverless, url, } = props;
    const [id, _] = useState(props.id || newUuid());
    const [adapter, setAdapter] = useState();
    const [extensionComponents, setExtensionComponents] = useState(new Array());
    const kernel = props.kernel ?? defaultKernel;
    const notebookStore = useNotebookStore();
    const portals = notebookStore.selectNotebookPortals(id);
    // Bootstrap the Notebook Adapter.
    const bootstrapAdapter = (serviceManager, kernel) => {
        const adapter = new NotebookAdapter({
            ...props,
            id,
            lite,
            kernel,
            serviceManager,
        });
        // Update the local state.
        setAdapter(adapter);
        extensions.forEach(extension => {
            extension.init({
                notebookId: id,
                commands: adapter.commands,
            });
            extension.createNew(adapter.notebookPanel, adapter.context);
            setExtensionComponents(extensionComponents.concat(extension.component ?? _jsx(_Fragment, {})));
        });
        // Update the global state.
        notebookStore.update({ id, state: { adapter } });
        // Update the global state based on events.
        adapter.notebookPanel?.model?.contentChanged.connect((notebookModel, _) => {
            notebookStore.changeModel({ id, notebookModel });
        });
        /*
        adapter.notebookPanel?.model!.sharedModel.changed.connect((_, notebookChange) => {
          notebookStore.notebookChange({ id, notebookChange });
        });
        adapter.notebookPanel?.content.modelChanged.connect((notebook, _) => {
          dispatÅch(notebookStore.notebookChange({ id, notebook }));
        });
        */
        adapter.notebookPanel?.content.activeCellChanged.connect((_, cellModel) => {
            if (cellModel === null) {
                notebookStore.activeCellChange({ id, cellModel: undefined });
            }
            else {
                notebookStore.activeCellChange({ id, cellModel });
            }
        });
        adapter.notebookPanel?.sessionContext.statusChanged.connect((_, kernelStatus) => {
            notebookStore.changeKernelStatus({ id, kernelStatus });
        });
        // Add more UI behavior when the Service Manager is ready...
        adapter.serviceManager.ready.then(() => {
            if (!readonly) {
                const cellModel = adapter.notebookPanel.content.activeCell;
                if (cellModel) {
                    notebookStore.activeCellChange({ id, cellModel });
                }
                const activeCellChanged$ = asObservable(adapter.notebookPanel.content.activeCellChanged);
                activeCellChanged$.subscribe((cellModel) => {
                    notebookStore.activeCellChange({ id, cellModel });
                    const panelDiv = document.getElementById('right-panel-id');
                    if (panelDiv) {
                        const cellMetadataOptions = (_jsx(Box, { mt: 3, children: _jsx(CellMetadataEditor, { notebookId: id, cell: cellModel, nbgrader: nbgrader }) }));
                        const portal = createPortal(cellMetadataOptions, panelDiv);
                        notebookStore.setPortalDisplay({ id, portalDisplay: { portal, pinned: false } });
                    }
                });
            }
        });
    };
    //
    const createAdapter = (serviceManager, kernel) => {
        if (!kernel) {
            bootstrapAdapter(serviceManager, kernel);
        }
        else {
            kernel.ready.then(() => {
                bootstrapAdapter(serviceManager, kernel);
            });
        }
    };
    const disposeAdapter = () => {
        adapter?.notebookPanel?.disposed.connect((slot, _) => {
            if (adapter) {
                notebookStore.dispose(id);
                setAdapter(undefined);
            }
        });
        adapter?.notebookPanel?.dispose();
    };
    // Mutation Effects.
    useEffect(() => {
        if (serviceManager && serverless) {
            createAdapter(serviceManager, kernel);
        }
        else if (serviceManager && kernel) {
            createAdapter(serviceManager, kernel);
        }
    }, [serviceManager, kernel]);
    useEffect(() => {
        if (adapter && adapter.kernel !== kernel) {
            adapter.setKernel(kernel);
        }
    }, [kernel]);
    useEffect(() => {
        if (adapter && adapter.readonly !== readonly) {
            adapter.setReadonly(readonly);
        }
    }, [readonly]);
    useEffect(() => {
        if (adapter && adapter.serverless !== serverless) {
            adapter.setServerless(serverless);
        }
    }, [serverless]);
    useEffect(() => {
        if (adapter && adapter.nbformat !== nbformat) {
            adapter.setNbformat(nbformat);
        }
    }, [nbformat]);
    useEffect(() => {
        if (adapter && path && adapter.path !== path) {
            disposeAdapter();
            createAdapter(serviceManager);
        }
    }, [path]);
    useEffect(() => {
        if (adapter && url && adapter.url !== url) {
            disposeAdapter();
            createAdapter(serviceManager);
        }
    }, [url]);
    // Dispose Effects.
    useEffect(() => {
        return () => {
            disposeAdapter();
        };
    }, []);
    //
    return (_jsxs(Box, { style: { height, width: '100%', position: 'relative' }, id: "dla-Jupyter-Notebook", children: [Toolbar && _jsx(Toolbar, { notebookId: id }), _jsxs(Box, { className: "dla-Box-Notebook", sx: {
                    '& .dla-Jupyter-Notebook': {
                        height,
                        maxHeight,
                        width: '100%',
                        overflowY: 'hidden',
                    },
                    '& .datalayer-NotebookPanel-header': {
                        minHeight: '50px',
                    },
                    '& .jp-Notebook': {
                        flex: '1 1 auto !important',
                        height: '100%',
                        overflowY: 'scroll',
                    },
                    '& .jp-NotebookPanel': {
                        height: '100% !important',
                        width: '100% !important',
                    },
                    '& .jp-Toolbar': {
                        display: 'none',
                        zIndex: 0,
                    },
                    '& .jp-Toolbar .jp-HTMLSelect.jp-DefaultStyle select': {
                        fontSize: '14px',
                    },
                    '& .jp-Toolbar > .jp-Toolbar-responsive-opener': {
                        display: 'none',
                    },
                    '& .jp-Toolbar-kernelName': {
                        display: 'none',
                    },
                    '& .jp-Cell': {
                        width: `calc(100% - ${props.cellSidebarMargin}px)`,
                    },
                    '& .jp-Notebook-footer': {
                        width: `calc(100% - ${props.cellSidebarMargin + 82}px)`,
                    },
                    '& .jp-Cell .jp-CellHeader': {
                        position: 'absolute',
                        top: '-5px',
                        left: `${props.cellSidebarMargin + 10}px`,
                        height: 'auto',
                    },
                    '& .jp-Cell .dla-CellSidebar-Container': {
                        padding: '4px 8px',
                        width: `${props.cellSidebarMargin + 10}px`,
                        marginLeft: 'auto',
                    },
                    '& .jp-CodeMirrorEditor': {
                        cursor: 'text !important',
                    },
                    '.dla-Box-Notebook': {
                        position: 'relative',
                    },
                }, children: [_jsx(_Fragment, { children: portals?.map((portal) => portal) }), _jsx(Box, { children: extensionComponents.map((extensionComponent, index) => {
                            return (_jsx(Box, { id: `${extensionComponent}-${index}`, children: extensionComponent }));
                        }) }), _jsx(Box, { children: adapter &&
                            _jsx(Lumino, { id: id, children: adapter.panel }) })] })] }));
};
Notebook.defaultProps = {
    cellMetadataPanel: false,
    cellSidebarMargin: 120,
    extensions: [],
    height: '100vh',
    maxHeight: '100vh',
    nbgrader: false,
    readonly: false,
    renderId: 0,
    renderers: [],
    serverless: false,
};
export default Notebook;
//# sourceMappingURL=Notebook.js.map